/*
    Valtra Tractors
    Copyright 2004 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Martin Storsj
    martin@martin.st
*/

#ifndef __GAME_H
#define __GAME_H

#include <SDL.h>
#include <vector>

using std::vector;

class Game {
protected:
	void (*updatedRect)(const SDL_Rect &r);
public:
	Game(SDL_Surface *screen, Uint32 time) {}
	virtual ~Game() {}
	virtual void handleEvent(SDL_Event &event) {}
	virtual void update(Uint32 time, SDL_Surface *screen) { }
	virtual bool finished() { return false; }
	virtual void setUpdatedRect(void (*func)(const SDL_Rect &r)) { updatedRect = func; }
	virtual void redraw() {}
	virtual void startGame(int players) {}
	static int addPlayer(int code, int player) {
		code |= 1<<(sizeof(int)*8-1);
		code |= 1<<player;
		return code;
	}
	static int removePlayer(int code, int player) {
		code &= ~(1<<player);
		return code;
	}
	static int getPlayers(int code) {
		if (code & (1<<(sizeof(int)*8-1))) {
			int n = 0;
			for (unsigned int i = 0; i < sizeof(int)*8-1; i++)
				if (code & (1<<i)) n++;
			return n;
		} else return code;
	}
	static bool hasPlayer(int code, int n) {
		if (code & (1<<(sizeof(int)*8-1))) {
			return (code & (1<<n)) > 0;
		} else {
			if (n<code) return true;
			return false;
		}
	}
	static bool isCoded(int code) {
		return (code & (1<<(sizeof(int)*8-1))) != 0;
	}
};

class GameWrapper : public Game {
private:
public:
	GameWrapper(SDL_Surface *screen, Uint32 time) : Game(screen,time) {}
	~GameWrapper() {}
	virtual void setPointer(int type) {}
	virtual void showQuitDialog() {}
};

class Game2 : public Game {
protected:
	GameWrapper *wrapper;
public:
	Game2(SDL_Surface *screen, Uint32 time) : Game(screen,time), wrapper(NULL) {}
	~Game2() {}
	virtual void pause(bool pause) {}
	virtual void getRects(vector<SDL_Rect> &list) {}
	virtual void setWrapper(GameWrapper *w) { wrapper = w; }
	virtual int getResult() { return 0; }
};



Game2 *constructBoardGame(SDL_Surface *screen, Uint32 time);
Game *constructWrapper(SDL_Surface *screen, Uint32 time);


#endif
